import { SUPPORTED_HLJS_LANGUAGES_ARRAY } from "../../../SupportedHLJSLanguages";
import { h } from "@stencil/core";
import DOMPurify from "dompurify";
import hljs from "highlight.js/lib/core";
import { marked } from "marked";
window.hljs = hljs;
DOMPurify.addHook('afterSanitizeAttributes', (node) => {
    if ('href' in node) {
        node.setAttribute('target', '_parent');
        node.setAttribute('rel', 'noopener');
    }
});
/**
 * In order to keep the bundle size small, we are loading the languages grammars by fecthing hljs's CDN dynamically.
 * This file is used as a in-memory static global variable to store the languages grammars references and fetching states.
 */
const SUPPORTED_HLJS_LANGUAGES = Object.fromEntries(SUPPORTED_HLJS_LANGUAGES_ARRAY.map((language) => [language, { error: false, retries: 0 }]));
// This is used for the cases the a known language returned by the LLM
// should be highlighted as another language
const LANGUAGES_MAPPING = {
    html: 'xml',
};
/**
 *
 * @param language the language to load
 * @returns true if the language is available either from beeing successfully loaded or from being already loaded
 */
async function loadLanguageAndHighlight(language) {
    // TODO: Supported languages may be overkill. Evaluate if we can remove this to make bundler slimmer
    const localCachedLanguage = SUPPORTED_HLJS_LANGUAGES[language];
    if (!localCachedLanguage) {
        // Unsupported language
        return false;
    }
    if (localCachedLanguage.error && localCachedLanguage.retries >= 5) {
        // Error too many retries
        return false;
    }
    try {
        const response = await fetch(`https://cdnjs.cloudflare.com/ajax/libs/highlight.js/11.10.0/languages/${language}.min.js`);
        if (!response.ok) {
            throw new Error(`Failed to fetch ${language} language: ${response.statusText}`);
        }
        const scriptContent = await response.text();
        // TODO: We way need to check if the script is safe
        const loadScript = new Function(scriptContent);
        // hljs language grammar from CDN automatically registers the language
        await loadScript();
        if (!hljs.getLanguage(language)) {
            throw new Error(`Even after executing grammar script, ${language} language was not registered`);
        }
        // Just for consistency
        localCachedLanguage.retries = 0;
        localCachedLanguage.error = false;
        return true;
    }
    catch (e) {
        localCachedLanguage.error = true;
        localCachedLanguage.retries++;
        return false;
    }
}
export class OramaMarkdown {
    constructor() {
        this.parseMarkdown = async () => {
            if (!this.content) {
                return;
            }
            // biome-ignore lint/suspicious/noMisleadingCharacterClass: No clear reason for this
            const noZeroWidthCharsContent = this.content.replace(/^[\u200B\u200C\u200D\u200E\u200F\uFEFF]/, '');
            const highlightedCode = await marked.parse(noZeroWidthCharsContent);
            this.divElement.innerHTML = DOMPurify.sanitize(highlightedCode);
            const mk = this.markdownElement.shadowRoot.querySelectorAll('.orama-markdown-code');
            const pedningBlocksByLanguage = {};
            for (let i = 0; i < mk.length; i++) {
                const codeBlockRef = mk[i];
                // Highlighted code blocks are marked with highlighted data attribute
                if (codeBlockRef.hasAttribute('data-highlighted')) {
                    continue;
                }
                const languageClassName = codeBlockRef.className.split(' ').find((className) => className.startsWith('language-'));
                const language = languageClassName.substring('language-'.length);
                pedningBlocksByLanguage[language] = true;
            }
            for (const pendingLanguage of Object.keys(pedningBlocksByLanguage)) {
                loadLanguageAndHighlight(pendingLanguage).then(() => {
                    // We know for sure that nothing is async right now
                    const parsedContent = marked.parse(noZeroWidthCharsContent);
                    this.divElement.innerHTML = DOMPurify.sanitize(parsedContent);
                });
            }
        };
    }
    onContentChange() {
        this.parseMarkdown();
    }
    handleMarkdownLinkClicked(originalOnClickEvent) {
        const htmlElement = originalOnClickEvent.target;
        const chatMarkdownLinkClicked = this.chatMarkdownLinkClicked.emit({
            text: htmlElement.innerText,
            href: htmlElement.href,
        });
        if (chatMarkdownLinkClicked.defaultPrevented) {
            originalOnClickEvent.preventDefault();
            return;
        }
    }
    componentDidLoad() {
        marked.use({
            useNewRenderer: true,
            renderer: {
                link: (token) => {
                    var _a, _b, _c, _d, _e;
                    const link = document.createElement('a');
                    link.innerHTML = (_b = (_a = this.chatMarkdownLinkTitle) === null || _a === void 0 ? void 0 : _a.call(this, { href: token.href, text: token.text })) !== null && _b !== void 0 ? _b : token.text;
                    link.href = (_d = (_c = this.chatMarkdownLinkHref) === null || _c === void 0 ? void 0 : _c.call(this, { href: token.href, text: token.text })) !== null && _d !== void 0 ? _d : token.href;
                    if (this.chatMarkdownLinkTarget) {
                        link.target = (_e = this.chatMarkdownLinkTarget) === null || _e === void 0 ? void 0 : _e.call(this, { href: token.href, text: token.text });
                    }
                    return link.outerHTML;
                },
                code: (token) => {
                    const mappedLanguage = LANGUAGES_MAPPING[token.lang] || token.lang;
                    // Some "languages" like HTML should be rendererd as XML. Reasons are still unclear.
                    const actualLanguage = SUPPORTED_HLJS_LANGUAGES[mappedLanguage] ? mappedLanguage : 'plaintext';
                    const pre = document.createElement('pre');
                    pre.classList.add('orama-markdown-pre');
                    const codeTitle = document.createElement('div');
                    codeTitle.classList.add('orama-markdown-code-title');
                    pre.appendChild(codeTitle);
                    const code = document.createElement('code');
                    code.classList.add('orama-markdown-code');
                    code.classList.add(`language-${actualLanguage}`);
                    code.classList.add('hljs');
                    pre.appendChild(code);
                    const hljsLanguage = hljs.getLanguage(actualLanguage);
                    if (hljsLanguage) {
                        if (actualLanguage === 'plaintext') {
                            codeTitle.innerHTML = '';
                        }
                        else {
                            codeTitle.innerHTML = hljsLanguage.name;
                        }
                        code.innerHTML = hljs.highlight(token.text, { language: actualLanguage, ignoreIllegals: true }).value;
                        code.dataset.highlighted = 'yes';
                    }
                    else {
                        codeTitle.innerHTML = ' ';
                        code.innerHTML = token.text;
                    }
                    return pre.outerHTML;
                },
                codespan: (token) => {
                    const code = document.createElement('code');
                    code.classList.add('orama-markdown-inline-code');
                    code.innerHTML = token.text;
                    return code.outerHTML;
                },
            },
        });
        this.parseMarkdown();
    }
    render() {
        return (h("host", { key: '0d9e85fa6fc920380df442ff1ef91e3fbcc0ea51' }, h("link", { key: '10de293b8a30b8071a00c819c6c31968d5f48804', rel: "stylesheet", href: "https://cdn.jsdelivr.net/gh/highlightjs/cdn-release@11.10.0/build/styles/atom-one-dark.min.css" }), h("div", { key: '610b83aa61a633c732e20331685c9e61cecd1a49', class: "orama-markdown-wrapper", ref: (ref) => {
                this.divElement = ref;
            }, onClick: (event) => {
                const htmlEvent = event.target;
                // Needs to be here because renderer function only outputs text. There is not a way to set a onclick there
                if (htmlEvent.tagName === 'A') {
                    this.handleMarkdownLinkClicked(event);
                }
            } })));
    }
    static get is() { return "orama-markdown"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["orama-markdown.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["orama-markdown.css"]
        };
    }
    static get properties() {
        return {
            "content": {
                "type": "string",
                "attribute": "content",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "chatMarkdownLinkTitle": {
                "type": "unknown",
                "attribute": "chat-markdown-link-title",
                "mutable": false,
                "complexType": {
                    "original": "ChatMarkdownLinkTitle",
                    "resolved": "({ text, href }: { text: string; href: string; }) => string",
                    "references": {
                        "ChatMarkdownLinkTitle": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ChatMarkdownLinkTitle"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "chatMarkdownLinkHref": {
                "type": "unknown",
                "attribute": "chat-markdown-link-href",
                "mutable": false,
                "complexType": {
                    "original": "ChatMarkdownLinkHref",
                    "resolved": "({ text, href }: { text: string; href: string; }) => string",
                    "references": {
                        "ChatMarkdownLinkHref": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ChatMarkdownLinkHref"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "chatMarkdownLinkTarget": {
                "type": "unknown",
                "attribute": "chat-markdown-link-target",
                "mutable": false,
                "complexType": {
                    "original": "ChatMarkdownLinkTarget",
                    "resolved": "({ text, href }: { text: string; href: string; }) => string",
                    "references": {
                        "ChatMarkdownLinkTarget": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ChatMarkdownLinkTarget"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            }
        };
    }
    static get events() {
        return [{
                "method": "chatMarkdownLinkClicked",
                "name": "chatMarkdownLinkClicked",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "complexType": {
                    "original": "OnChatMarkdownLinkClickedCallbackProps",
                    "resolved": "{ text: string; href: string; }",
                    "references": {
                        "OnChatMarkdownLinkClickedCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::OnChatMarkdownLinkClickedCallbackProps"
                        }
                    }
                }
            }];
    }
    static get elementRef() { return "markdownElement"; }
    static get watchers() {
        return [{
                "propName": "content",
                "methodName": "onContentChange"
            }];
    }
}
//# sourceMappingURL=orama-markdown.js.map
