import { Host, h } from "@stencil/core";
import { ChatService } from "../../services/ChatService";
import { generateRandomID, initOramaClient, updateCssVariables, updateThemeClasses, validateCloudIndexConfig, } from "../../utils/utils";
import { getText as getTextUtil } from "../../utils/textDictionary";
import "@phosphor-icons/webcomponents/dist/icons/PhArrowClockwise.mjs";
import { initStore, removeAllStores } from "../../ParentComponentStore/ParentComponentStoreManager";
export class ChatBox {
    constructor() {
        this.autoFocus = true;
        this.clearChatOnDisconnect = true;
        /**
         * Text dictionary for customizing all text content in the component.
         * This can be set either via HTML attribute as a JSON string or via JavaScript as an object.
         * It allows for customization of all text elements like placeholders, error messages, and UI labels.
         * @example
         * // Via HTML attribute
         * <orama-chat-box dictionary='{"chatPlaceholder": "Ask about our docs..."}' />
         *
         * // Via JavaScript
         * const chatBox = document.querySelector('orama-chat-box');
         * chatBox.dictionary = { chatPlaceholder: "Ask about our docs..." };
         */
        this.dictionary = {};
        /**
         * Component color schema
         */
        this.colorScheme = 'light';
        this.componentID = generateRandomID('chat-box');
        this.systemScheme = 'light';
        this.onPrefersColorSchemeChange = (event) => {
            this.systemScheme = event.matches ? 'dark' : 'light';
            this.updateTheme();
        };
    }
    /**
     * Gets the text for a specific key from the dictionary prop.
     * Prioritizes direct props (placeholder) for backward compatibility,
     * then falls back to the dictionary prop, and finally to the defaultTextDictionary.
     *
     * @param key - The key to get the text forstartChatService
     * @returns The text for the specified key
     */
    getText(key) {
        // Create a map of direct props for backward compatibility
        const directProps = {
            chatPlaceholder: this.placeholder,
        };
        return getTextUtil(key, this.dictionary, directProps);
    }
    watchHandler() {
        // This is a naive way to check if it is safe to eval this method (after componentWillLoad)
        if (!this.chatStore) {
            return;
        }
        this.startChatService();
        this.updateTheme();
    }
    /**
     * Watch for changes to the dictionary prop
     */
    handleTextDictionaryChange(newValue) {
        // Handle case where dictionary is passed as a string (via HTML attribute)
        if (typeof newValue === 'string') {
            try {
                this.dictionary = JSON.parse(newValue);
            }
            catch (e) {
                console.error('Error parsing dictionary:', e);
            }
        }
    }
    componentWillLoad() {
        this.htmlElement.id = this.componentID;
        this.schemaQuery = window.matchMedia('(prefers-color-scheme: dark)');
        this.systemScheme = this.schemaQuery.matches ? 'dark' : 'light';
        this.updateTheme();
        this.schemaQuery.addEventListener('change', this.onPrefersColorSchemeChange);
        this.startChatService();
    }
    componentDidLoad() {
        this.htmlElement.id = this.componentID;
        this.startChatService();
    }
    /**
     * Get the old Orama Client if oramaCore ins't available
     */
    getOldOramaClient() {
        if (this.oramaCoreClientInstance) {
            return undefined;
        }
        if (this.clientInstance) {
            return this.clientInstance;
        }
        return initOramaClient(this.index);
    }
    startChatService() {
        if (!this.index && !this.clientInstance && !this.oramaCoreClientInstance) {
            console.error('Missing Index, ClientInstance or OramaCoreClientInstance');
            // Skip initialization if no index or clientInstance is provided
            return;
        }
        validateCloudIndexConfig(this.htmlElement, this.index, this.clientInstance, this.oramaCoreClientInstance);
        const oldOramaClient = this.getOldOramaClient();
        this.chatStore.state.chatService = new ChatService(oldOramaClient, this.oramaCoreClientInstance, this.chatStore);
        if (!this.chatStore.state.chatService) {
            console.error('Failed to initialize chat service');
        }
    }
    updateTheme() {
        const scheme = updateThemeClasses(this.htmlElement, this.colorScheme, this.systemScheme);
        updateCssVariables(this.htmlElement, scheme, this.themeConfig);
    }
    connectedCallback() {
        this.chatStore = initStore('chat', this.componentID);
    }
    disconnectedCallback() {
        var _a;
        removeAllStores(this.componentID);
        (_a = this.schemaQuery) === null || _a === void 0 ? void 0 : _a.removeEventListener('change', this.onPrefersColorSchemeChange);
    }
    render() {
        var _a, _b;
        if (!this.chatStore.state.chatService) {
            return null;
        }
        // Maintain backwards compatibility with dark theme default
        const defaultTheme = 'dark';
        const currentScheme = this.colorScheme === 'system' ? this.systemScheme : this.colorScheme || defaultTheme;
        return (h(Host, { class: `theme-${currentScheme}` }, h("orama-chat", { placeholder: this.getText('chatPlaceholder'), sourceBaseUrl: this.sourceBaseUrl, sourcesMap: this.sourcesMap, suggestions: this.suggestions, relatedQueries: this.relatedQueries, focusInput: this.autoFocus, systemPrompts: this.systemPrompts, prompt: this.prompt, clearChatOnDisconnect: this.clearChatOnDisconnect, chatMarkdownLinkTitle: this.chatMarkdownLinkTitle, chatMarkdownLinkHref: this.chatMarkdownLinkHref, dictionary: this.dictionary, disclaimer: this.disclaimer }, !!((_b = (_a = this.chatStore.state) === null || _a === void 0 ? void 0 : _a.interactions) === null || _b === void 0 ? void 0 : _b.length) && (h("div", { slot: "chat-empty-state" }, h("slot", { name: "empty-state" }))))));
    }
    static get is() { return "orama-chat-box"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["orama-chat-box.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["orama-chat-box.css"]
        };
    }
    static get properties() {
        return {
            "index": {
                "type": "unknown",
                "attribute": "index",
                "mutable": false,
                "complexType": {
                    "original": "CloudIndexConfig | CloudIndexConfig[]",
                    "resolved": "CloudIndexConfig[] | { api_key: string; endpoint: string; }",
                    "references": {
                        "CloudIndexConfig": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::CloudIndexConfig"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "clientInstance": {
                "type": "unknown",
                "attribute": "client-instance",
                "mutable": false,
                "complexType": {
                    "original": "OramaClient | AnyOrama",
                    "resolved": "FunctionComponents<any> & Internals<any, AnyIndexStore, AnyDocumentStore, AnySorterStore> & ArrayCallbackComponents<any> & OramaID & { plugins: OramaPlugin<unknown>[]; } | OramaClient<true>",
                    "references": {
                        "OramaClient": {
                            "location": "import",
                            "path": "@oramacloud/client",
                            "id": "../../node_modules/.pnpm/@oramacloud+client@2.1.4/node_modules/@oramacloud/client/dist/index.d.ts::OramaClient"
                        },
                        "AnyOrama": {
                            "location": "import",
                            "path": "@orama/orama",
                            "id": "../../node_modules/.pnpm/@orama+orama@3.1.6/node_modules/@orama/orama/dist/commonjs/index.d.ts::AnyOrama"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "oramaCoreClientInstance": {
                "type": "unknown",
                "attribute": "orama-core-client-instance",
                "mutable": false,
                "complexType": {
                    "original": "CollectionManager",
                    "resolved": "CollectionManager",
                    "references": {
                        "CollectionManager": {
                            "location": "import",
                            "path": "@orama/core",
                            "id": "../../node_modules/.pnpm/@orama+core@0.1.8/node_modules/@orama/core/script/index.d.ts::CollectionManager"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "sourceBaseUrl": {
                "type": "string",
                "attribute": "source-base-url",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "linksTarget": {
                "type": "string",
                "attribute": "links-target",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "linksRel": {
                "type": "string",
                "attribute": "links-rel",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "placeholder": {
                "type": "string",
                "attribute": "placeholder",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "sourcesMap": {
                "type": "unknown",
                "attribute": "sources-map",
                "mutable": false,
                "complexType": {
                    "original": "SourcesMap",
                    "resolved": "SourcesMapItem[] | { path?: string | NewType; description?: string | NewType; title?: string | NewType; } & { datasourceId?: string; }",
                    "references": {
                        "SourcesMap": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::SourcesMap"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "suggestions": {
                "type": "unknown",
                "attribute": "suggestions",
                "mutable": false,
                "complexType": {
                    "original": "string[]",
                    "resolved": "string[]",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "relatedQueries": {
                "type": "number",
                "attribute": "related-queries",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "autoFocus": {
                "type": "boolean",
                "attribute": "auto-focus",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "true"
            },
            "systemPrompts": {
                "type": "unknown",
                "attribute": "system-prompts",
                "mutable": false,
                "complexType": {
                    "original": "string[]",
                    "resolved": "string[]",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "prompt": {
                "type": "string",
                "attribute": "prompt",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "clearChatOnDisconnect": {
                "type": "boolean",
                "attribute": "clear-chat-on-disconnect",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "true"
            },
            "chatMarkdownLinkTitle": {
                "type": "unknown",
                "attribute": "chat-markdown-link-title",
                "mutable": false,
                "complexType": {
                    "original": "ChatMarkdownLinkTitle",
                    "resolved": "({ text, href }: { text: string; href: string; }) => string",
                    "references": {
                        "ChatMarkdownLinkTitle": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ChatMarkdownLinkTitle"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "chatMarkdownLinkHref": {
                "type": "unknown",
                "attribute": "chat-markdown-link-href",
                "mutable": false,
                "complexType": {
                    "original": "ChatMarkdownLinkHref",
                    "resolved": "({ text, href }: { text: string; href: string; }) => string",
                    "references": {
                        "ChatMarkdownLinkHref": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ChatMarkdownLinkHref"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "chatMarkdownLinkTarget": {
                "type": "unknown",
                "attribute": "chat-markdown-link-target",
                "mutable": false,
                "complexType": {
                    "original": "ChatMarkdownLinkTarget",
                    "resolved": "({ text, href }: { text: string; href: string; }) => string",
                    "references": {
                        "ChatMarkdownLinkTarget": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ChatMarkdownLinkTarget"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "dictionary": {
                "type": "unknown",
                "attribute": "dictionary",
                "mutable": false,
                "complexType": {
                    "original": "Partial<Dictionary>",
                    "resolved": "{ searchPlaceholder?: string; chatPlaceholder?: string; noResultsFound?: string; noResultsFoundFor?: string; suggestions?: string; seeAll?: string; addMore?: string; clearChat?: string; errorMessage?: string; disclaimer?: string; startYourSearch?: string; initErrorSearch?: string; initErrorChat?: string; chatButtonLabel?: string; searchButtonLabel?: string; }",
                    "references": {
                        "Partial": {
                            "location": "global",
                            "id": "global::Partial"
                        },
                        "Dictionary": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::Dictionary"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "example",
                            "text": "// Via HTML attribute\n<orama-chat-box dictionary='{\"chatPlaceholder\": \"Ask about our docs...\"}' />\n\n// Via JavaScript\nconst chatBox = document.querySelector('orama-chat-box');\nchatBox.dictionary = { chatPlaceholder: \"Ask about our docs...\" };"
                        }],
                    "text": "Text dictionary for customizing all text content in the component.\nThis can be set either via HTML attribute as a JSON string or via JavaScript as an object.\nIt allows for customization of all text elements like placeholders, error messages, and UI labels."
                },
                "getter": false,
                "setter": false,
                "defaultValue": "{}"
            },
            "disclaimer": {
                "type": "string",
                "attribute": "disclaimer",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "themeConfig": {
                "type": "unknown",
                "attribute": "theme-config",
                "mutable": false,
                "complexType": {
                    "original": "Partial<TThemeOverrides>",
                    "resolved": "{ typography?: DeepPartial<{ '--font-primary': string; }>; colors?: DeepPartial<{ gray50: string; gray100: string; gray200: string; gray300: string; gray400: string; gray500: string; gray600: string; gray700: string; gray800: string; gray900: string; gray950: string; purple100: string; purple200: string; purple300: string; purple500: string; purple600: string; purple700: string; light: { \"--text-color-primary\": string; \"--text-color-secondary\": string; \"--text-color-tertiary\": string; \"--text-color-accent\": string; \"--text-color-inactive\": string; \"--text-color-reverse\": string; \"--background-color-primary\": string; \"--background-color-secondary\": string; \"--background-color-tertiary\": string; \"--background-color-fourth\": string; \"--background-color-reverse\": string; \"--background-color-accent\": string; \"--border-color-primary\": string; \"--border-color-secondary\": string; \"--border-color-tertiary\": string; \"--border-color-accent\": string; \"--icon-color-primary\": string; \"--icon-color-secondary\": string; \"--icon-color-tertiary\": string; \"--icon-color-inactive\": string; \"--icon-color-reverse\": string; \"--shadow-color-primary\": string; \"--button-text-color-primary\": string; \"--button-text-color-secondary\": string; \"--button-background-color-primary\": string; \"--button-background-color-secondary\": string; \"--button-background-color-secondary-hover\": string; \"--button-border-color-secondary\": string; \"--backdrop-background-color-primary\": string; \"--chat-button-border-color-gradientOne\": string; \"--chat-button-border-color-gradientTwo\": string; \"--chat-button-border-color-gradientThree\": string; \"--chat-button-border-color-gradientFour\": string; \"--chat-button-border-color-gradientFive\": string; \"--chat-button-border-color-gradientSix\": string; \"--chat-button-background-color-gradientOne\": string; \"--chat-button-background-color-gradientTwo\": string; }; dark: { \"--text-color-primary\": string; \"--text-color-secondary\": string; \"--text-color-tertiary\": string; \"--text-color-accent\": string; \"--text-color-inactive\": string; \"--text-color-reverse\": string; \"--background-color-primary\": string; \"--background-color-secondary\": string; \"--background-color-tertiary\": string; \"--background-color-fourth\": string; \"--background-color-reverse\": string; \"--background-color-accent\": string; \"--border-color-primary\": string; \"--border-color-secondary\": string; \"--border-color-tertiary\": string; \"--border-color-accent\": string; \"--icon-color-primary\": string; \"--icon-color-secondary\": string; \"--icon-color-tertiary\": string; \"--icon-color-inactive\": string; \"--icon-color-reverse\": string; \"--shadow-color-primary\": string; \"--button-text-color-primary\": string; \"--button-text-color-secondary\": string; \"--button-background-color-primary\": string; \"--button-background-color-secondary\": string; \"--button-background-color-secondary-hover\": string; \"--button-border-color-secondary\": string; \"--backdrop-background-color-primary\": string; \"--chat-button-border-color-gradientOne\": string; \"--chat-button-border-color-gradientTwo\": string; \"--chat-button-border-color-gradientThree\": string; \"--chat-button-border-color-gradientFour\": string; \"--chat-button-border-color-gradientFive\": string; \"--chat-button-border-color-gradientSix\": string; \"--chat-button-background-color-gradientOne\": string; \"--chat-button-background-color-gradientTwo\": string; }; }>; radius?: DeepPartial<{ '--radius-xs': string; '--radius-s': string; '--radius-m': string; '--radius-l': string; '--radius-3xl': string; '--textarea-radius': string; }>; shadow?: DeepPartial<{ '--textarea-shadow': any; }>; }",
                    "references": {
                        "Partial": {
                            "location": "global",
                            "id": "global::Partial"
                        },
                        "TThemeOverrides": {
                            "location": "import",
                            "path": "@/config/theme",
                            "id": "src/config/theme.ts::TThemeOverrides"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Component theme customization"
                },
                "getter": false,
                "setter": false
            },
            "colorScheme": {
                "type": "string",
                "attribute": "color-scheme",
                "mutable": false,
                "complexType": {
                    "original": "ColorScheme",
                    "resolved": "\"dark\" | \"light\" | \"system\"",
                    "references": {
                        "ColorScheme": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ColorScheme"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Component color schema"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "'light'"
            }
        };
    }
    static get states() {
        return {
            "componentID": {},
            "systemScheme": {}
        };
    }
    static get events() {
        return [{
                "method": "answerGenerated",
                "name": "answerGenerated",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when answer generation is successfully completed"
                },
                "complexType": {
                    "original": "OnAnswerGeneratedCallbackProps",
                    "resolved": "{ askParams: AskParams; query: string; sources: Results<unknown>; answer: string; segment: string; trigger: string; }",
                    "references": {
                        "OnAnswerGeneratedCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::OnAnswerGeneratedCallbackProps"
                        }
                    }
                }
            }, {
                "method": "clearChat",
                "name": "clearChat",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when the chat is cleared"
                },
                "complexType": {
                    "original": "void",
                    "resolved": "void",
                    "references": {}
                }
            }, {
                "method": "startConversation",
                "name": "startConversation",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired as soon as the conversation is started"
                },
                "complexType": {
                    "original": "onStartConversationCallbackProps",
                    "resolved": "{ userPrompt: string; systemPrompts: string[]; }",
                    "references": {
                        "onStartConversationCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::onStartConversationCallbackProps"
                        }
                    }
                }
            }, {
                "method": "answerSourceClick",
                "name": "answerSourceClick",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when user clicks on answer source"
                },
                "complexType": {
                    "original": "OnAnswerSourceClickCallbackProps",
                    "resolved": "{ source: SearchResult; }",
                    "references": {
                        "OnAnswerSourceClickCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::OnAnswerSourceClickCallbackProps"
                        }
                    }
                }
            }, {
                "method": "chatMarkdownLinkClicked",
                "name": "chatMarkdownLinkClicked",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when user clicks on chat markdown link"
                },
                "complexType": {
                    "original": "OnChatMarkdownLinkClickedCallbackProps",
                    "resolved": "{ text: string; href: string; }",
                    "references": {
                        "OnChatMarkdownLinkClickedCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::OnChatMarkdownLinkClickedCallbackProps"
                        }
                    }
                }
            }];
    }
    static get elementRef() { return "htmlElement"; }
    static get watchers() {
        return [{
                "propName": "index",
                "methodName": "watchHandler"
            }, {
                "propName": "themeConfig",
                "methodName": "watchHandler"
            }, {
                "propName": "colorScheme",
                "methodName": "watchHandler"
            }, {
                "propName": "oramaCoreClientInstance",
                "methodName": "watchHandler"
            }, {
                "propName": "clientInstance",
                "methodName": "watchHandler"
            }, {
                "propName": "dictionary",
                "methodName": "handleTextDictionaryChange"
            }];
    }
}
//# sourceMappingURL=orama-chat-box.js.map
