import { h, Fragment } from "@stencil/core";
import { ChatService } from "../../services/ChatService";
import { SearchService } from "../../services/SearchService";
import { windowWidthListener } from "../../services/WindowService";
import { arrowKeysNavigation, generateRandomID, initOramaClient, updateCssVariables, updateThemeClasses, validateCloudIndexConfig, } from "../../utils/utils";
import { defaultTextDictionary as importedDefaultTextDictionary, getText as getTextUtil } from "../../utils/textDictionary";
import { initStore, removeAllStores } from "../../ParentComponentStore/ParentComponentStoreManager";
// TODO: AI components should be lazyly loaded. In case of Disable AI flag, it should not be loaded at all
// https://linear.app/oramasearch/issue/ORM-1824/ai-components-should-be-lazyly-loaded-in-case-of-disable-ai-flag-they
export class SearchBox {
    constructor() {
        /**
         * Component color schema
         */
        this.colorScheme = 'light';
        this.open = false;
        /**
         * Used to map dataset result properties to the expected SearchBox properties
         */
        this.resultMap = {};
        /**
         * Disables chat capabilities
         */
        this.disableChat = false;
        /**
         * This component can behave either as Modal or a Embed component.
         * For Modal, a new absolute panel will be displayed on top.
         * For Embed, Orama Search Box will be displayed as a inline component.
         */
        this.layout = 'modal';
        /**
         * Options for highlights of Search Result titles
         */
        this.highlightTitle = false;
        /**
         * Options for highlights of Search Result descriptions
         */
        this.highlightDescription = false;
        /**
         * Clear chat on disconnect
         */
        this.clearChatOnDisconnect = true;
        /**
         * Show keyboard shortcuts in the footer
         */
        this.showKeyboardShortcuts = true;
        /**
         * Custom text dictionary for localization
         * @example
         * const searchBox = document.querySelector('orama-search-box');
         * searchBox.dictionary = { searchPlaceholder: "Search our products..." };
         */
        this.dictionary = {};
        this.componentID = generateRandomID('search-box');
        this.systemScheme = 'light';
        this.closeSearchbox = () => {
            this.globalStore.state.open = false;
            this.open = false;
        };
        this.onChatButtonClick = () => {
            this.globalStore.state.currentTask = 'chat';
        };
        this.onPrefersColorSchemeChange = (event) => {
            this.systemScheme = event.matches ? 'dark' : 'light';
            this.updateTheme();
        };
        this.updateWindowWidth = (event) => {
            this.windowWidth = event.detail;
        };
    }
    /**
     * Watch for changes to the dictionary prop
     */
    handleDictionaryChange(newValue) {
        let parsedNewValue = {};
        if (typeof newValue === 'string') {
            try {
                parsedNewValue = JSON.parse(newValue);
            }
            catch (e) {
                console.error('Failed to parse dictionary string:', e);
                // Keep current internalTextDictionary or revert to pure defaults if preferred
            }
        }
        else {
            parsedNewValue = newValue || {};
        }
        this.internalTextDictionary = Object.assign(Object.assign({}, importedDefaultTextDictionary), parsedNewValue);
    }
    indexChanged() {
        // This is a naive way to check if it is safe to eval this method (after componentWillLoad)
        if (!this.searchStore) {
            return;
        }
        this.bootstrap();
    }
    watchHandler() {
        this.updateTheme();
    }
    handleOpenPropChange(newValue) {
        this.globalStore.state.open = newValue;
        this.modalStatusChanged.emit({
            open: newValue,
            id: this.wrapperRef,
        });
        if (!newValue) {
            this.modalClosed.emit();
        }
    }
    handleFacetPropertyChange(newValue) {
        this.searchStore.state.facetProperty = newValue;
    }
    handleSearchParamsChange(newValue) {
        this.searchStore.state.searchParams = newValue;
    }
    handleKeyDown(ev) {
        var _a;
        if (((_a = this.globalStore) === null || _a === void 0 ? void 0 : _a.state.currentTask) === 'search' &&
            ((this.layout === 'modal' && this.open) || this.layout === 'embed') &&
            ['ArrowDown', 'ArrowUp'].includes(ev.key)) {
            arrowKeysNavigation(this.wrapperRef, ev);
        }
    }
    updateTheme() {
        const scheme = updateThemeClasses(this.htmlElement, this.colorScheme, this.systemScheme);
        updateCssVariables(this.htmlElement, scheme, this.themeConfig);
    }
    getOldOramaClient() {
        if (this.oramaCoreClientInstance) {
            return undefined;
        }
        if (this.clientInstance) {
            return this.clientInstance;
        }
        return initOramaClient(this.index);
    }
    bootstrap() {
        this.searchStore.state.facetProperty = this.facetProperty;
        this.searchStore.state.resultMap = this.resultMap;
        this.searchStore.state.searchParams = this.searchParams;
        this.startServices();
    }
    startServices() {
        if (!this.index && !this.clientInstance && !this.oramaCoreClientInstance) {
            // Skip initialization if no index or clientInstance is provided
            return;
        }
        validateCloudIndexConfig(this.htmlElement, this.index, this.clientInstance, this.oramaCoreClientInstance);
        const oldOramaClient = this.getOldOramaClient();
        this.searchStore.state.searchService = new SearchService(oldOramaClient, this.oramaCoreClientInstance, this.searchStore);
        this.chatStore.state.chatService = new ChatService(oldOramaClient, this.oramaCoreClientInstance, this.chatStore);
    }
    componentWillLoad() {
        this.htmlElement.id = this.componentID;
        this.bootstrap();
        this.globalStore.state.open = this.open;
        this.globalStore.onChange('open', () => {
            if (!this.globalStore) {
                return;
            }
            this.open = this.globalStore.state.open;
            if (!this.globalStore.state.open) {
                this.globalStore.state.currentTerm = '';
                // TODO: We should be reseting the context, but we do not want to lose params definitions.
                // We may want to handle params in a different way.
                this.searchStore.state.facets = [];
                this.searchStore.state.count = 0;
                this.searchStore.state.results = [];
                this.searchStore.state.highlightedIndex = -1;
                this.searchStore.state.loading = false;
                this.searchStore.state.error = false;
                this.chatStore.state.interactions = [];
                setTimeout(() => { });
            }
        });
        this.schemaQuery = window.matchMedia('(prefers-color-scheme: dark)');
        this.systemScheme = this.schemaQuery.matches ? 'dark' : 'light';
        this.updateTheme();
        this.schemaQuery.addEventListener('change', this.onPrefersColorSchemeChange);
        windowWidthListener.addEventListener('widthChange', this.updateWindowWidth);
        let propValue = {};
        if (typeof this.dictionary === 'string') {
            try {
                propValue = JSON.parse(this.dictionary);
            }
            catch (e) {
                console.error('Failed to parse initial dictionary string:', e);
            }
        }
        else {
            propValue = this.dictionary || {};
        }
        this.internalTextDictionary = Object.assign(Object.assign({}, importedDefaultTextDictionary), propValue);
    }
    connectedCallback() {
        this.chatStore = initStore('chat', this.componentID);
        this.searchStore = initStore('search', this.componentID);
        this.globalStore = initStore('global', this.componentID);
        this.windowWidth = windowWidthListener.width;
    }
    disconnectedCallback() {
        var _a;
        removeAllStores(this.componentID);
        windowWidthListener.removeEventListener('widthChange', this.updateWindowWidth);
        (_a = this.schemaQuery) === null || _a === void 0 ? void 0 : _a.removeEventListener('change', this.onPrefersColorSchemeChange);
    }
    /**
     * Gets the text for a specific key from the dictionary prop.
     * Prioritizes direct props (searchPlaceholder, chatPlaceholder) for backward compatibility,
     * then falls back to the dictionary prop, and finally to the defaultTextDictionary.
     *
     * @param key - The key to get the text for
     * @returns The text for the specified key
     */
    getText(key) {
        // Create a map of direct props for backward compatibility
        const directProps = {
            searchPlaceholder: this.searchPlaceholder,
            chatPlaceholder: this.chatPlaceholder,
        };
        return getTextUtil(key, this.internalTextDictionary, directProps);
    }
    getSearchBox() {
        return (h("div", { class: `${this.windowWidth > 1024
                ? 'section-active'
                : this.globalStore.state.currentTask === 'search'
                    ? 'section-active'
                    : 'section-inactive'}` }, h("orama-search", { placeholder: this.getText('searchPlaceholder'), focusInput: this.globalStore.state.currentTask === 'search', sourceBaseUrl: this.sourceBaseUrl, linksTarget: this.linksTarget, linksRel: this.linksRel, highlightTitle: this.highlightTitle, highlightDescription: this.highlightDescription, disableChat: this.disableChat, suggestions: this.suggestions, dictionary: this.internalTextDictionary }, this.windowWidth > 1024 && !this.disableChat && (h("orama-chat-button", { slot: "summary", "focus-on-arrow-nav": true, active: !!this.globalStore.state.currentTerm, label: `${this.globalStore.state.currentTerm ? `${this.globalStore.state.currentTerm} - ` : ''}${this.getText('chatButtonLabel')}`, onClick: this.onChatButtonClick, onKeyPress: this.onChatButtonClick })))));
    }
    getChatBox() {
        return (h(Fragment, null, h("orama-chat", { class: `${this.globalStore.state.currentTask === 'chat' ? 'section-active' : 'section-inactive'}`, defaultTerm: this.globalStore.state.currentTask === 'chat' ? this.globalStore.state.currentTerm : '', showClearChat: false, focusInput: this.globalStore.state.currentTask === 'chat', placeholder: this.getText('chatPlaceholder'), sourceBaseUrl: this.sourceBaseUrl, linksTarget: this.linksTarget, linksRel: this.linksRel, sourcesMap: this.sourcesMap, suggestions: this.suggestions, chatMarkdownLinkTitle: this.chatMarkdownLinkTitle, chatMarkdownLinkHref: this.chatMarkdownLinkHref, chatMarkdownLinkTarget: this.chatMarkdownLinkTarget, disclaimer: this.disclaimer, dictionary: this.internalTextDictionary, systemPrompts: this.systemPrompts, prompt: this.prompt, clearChatOnDisconnect: this.clearChatOnDisconnect, relatedQueries: this.relatedQueries })));
    }
    getInnerContent() {
        return (h(Fragment, null, this.disableChat ? null : (h("orama-navigation-bar", { handleClose: this.closeSearchbox, showBackButton: this.layout !== 'embed', showChatActions: this.globalStore.state.currentTask === 'chat' })), h("div", { class: "main" }, this.getSearchBox(), this.windowWidth <= 1024 && this.getChatBox()), h("orama-footer", { colorScheme: this.colorScheme === 'system' ? this.systemScheme : this.colorScheme, showKeyboardShortcuts: this.showKeyboardShortcuts })));
    }
    getOuterContent() {
        return this.windowWidth > 1024 ? (h("orama-sliding-panel", { open: this.globalStore.state.currentTask === 'chat', backdrop: this.layout === 'embed', closed: () => {
                this.globalStore.state.currentTask = 'search';
            } }, this.getChatBox())) : null;
    }
    getModalLayout() {
        return (h(Fragment, null, h("orama-modal", { ref: (el) => (this.wrapperRef = el), class: "modal", mainTitle: this.getText('startYourSearch'), onModalClosed: (e) => {
                this.open = false;
                e.stopPropagation();
            }, closeOnEscape: this.globalStore.state.currentTask === 'search' || this.windowWidth <= 1024 }, this.getInnerContent()), this.getOuterContent()));
    }
    getEmbedLayout() {
        return (h(Fragment, null, h("orama-embed", { ref: (el) => (this.wrapperRef = el) }, this.getInnerContent()), this.getOuterContent()));
    }
    render() {
        if (this.layout === 'modal' && !this.globalStore.state.open) {
            return null;
        }
        if (!this.searchStore.state.searchService) {
            return null;
        }
        if (!this.chatStore.state.chatService) {
            return null;
        }
        return this.layout === 'modal' ? this.getModalLayout() : this.getEmbedLayout();
    }
    static get is() { return "orama-search-box"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["orama-search-box.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["orama-search-box.css"]
        };
    }
    static get properties() {
        return {
            "themeConfig": {
                "type": "unknown",
                "attribute": "theme-config",
                "mutable": false,
                "complexType": {
                    "original": "Partial<TThemeOverrides>",
                    "resolved": "{ typography?: DeepPartial<{ '--font-primary': string; }>; colors?: DeepPartial<{ gray50: string; gray100: string; gray200: string; gray300: string; gray400: string; gray500: string; gray600: string; gray700: string; gray800: string; gray900: string; gray950: string; purple100: string; purple200: string; purple300: string; purple500: string; purple600: string; purple700: string; light: { \"--text-color-primary\": string; \"--text-color-secondary\": string; \"--text-color-tertiary\": string; \"--text-color-accent\": string; \"--text-color-inactive\": string; \"--text-color-reverse\": string; \"--background-color-primary\": string; \"--background-color-secondary\": string; \"--background-color-tertiary\": string; \"--background-color-fourth\": string; \"--background-color-reverse\": string; \"--background-color-accent\": string; \"--border-color-primary\": string; \"--border-color-secondary\": string; \"--border-color-tertiary\": string; \"--border-color-accent\": string; \"--icon-color-primary\": string; \"--icon-color-secondary\": string; \"--icon-color-tertiary\": string; \"--icon-color-inactive\": string; \"--icon-color-reverse\": string; \"--shadow-color-primary\": string; \"--button-text-color-primary\": string; \"--button-text-color-secondary\": string; \"--button-background-color-primary\": string; \"--button-background-color-secondary\": string; \"--button-background-color-secondary-hover\": string; \"--button-border-color-secondary\": string; \"--backdrop-background-color-primary\": string; \"--chat-button-border-color-gradientOne\": string; \"--chat-button-border-color-gradientTwo\": string; \"--chat-button-border-color-gradientThree\": string; \"--chat-button-border-color-gradientFour\": string; \"--chat-button-border-color-gradientFive\": string; \"--chat-button-border-color-gradientSix\": string; \"--chat-button-background-color-gradientOne\": string; \"--chat-button-background-color-gradientTwo\": string; }; dark: { \"--text-color-primary\": string; \"--text-color-secondary\": string; \"--text-color-tertiary\": string; \"--text-color-accent\": string; \"--text-color-inactive\": string; \"--text-color-reverse\": string; \"--background-color-primary\": string; \"--background-color-secondary\": string; \"--background-color-tertiary\": string; \"--background-color-fourth\": string; \"--background-color-reverse\": string; \"--background-color-accent\": string; \"--border-color-primary\": string; \"--border-color-secondary\": string; \"--border-color-tertiary\": string; \"--border-color-accent\": string; \"--icon-color-primary\": string; \"--icon-color-secondary\": string; \"--icon-color-tertiary\": string; \"--icon-color-inactive\": string; \"--icon-color-reverse\": string; \"--shadow-color-primary\": string; \"--button-text-color-primary\": string; \"--button-text-color-secondary\": string; \"--button-background-color-primary\": string; \"--button-background-color-secondary\": string; \"--button-background-color-secondary-hover\": string; \"--button-border-color-secondary\": string; \"--backdrop-background-color-primary\": string; \"--chat-button-border-color-gradientOne\": string; \"--chat-button-border-color-gradientTwo\": string; \"--chat-button-border-color-gradientThree\": string; \"--chat-button-border-color-gradientFour\": string; \"--chat-button-border-color-gradientFive\": string; \"--chat-button-border-color-gradientSix\": string; \"--chat-button-background-color-gradientOne\": string; \"--chat-button-background-color-gradientTwo\": string; }; }>; radius?: DeepPartial<{ '--radius-xs': string; '--radius-s': string; '--radius-m': string; '--radius-l': string; '--radius-3xl': string; '--textarea-radius': string; }>; shadow?: DeepPartial<{ '--textarea-shadow': any; }>; }",
                    "references": {
                        "Partial": {
                            "location": "global",
                            "id": "global::Partial"
                        },
                        "TThemeOverrides": {
                            "location": "import",
                            "path": "@/config/theme",
                            "id": "src/config/theme.ts::TThemeOverrides"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Component theme customization"
                },
                "getter": false,
                "setter": false
            },
            "colorScheme": {
                "type": "string",
                "attribute": "color-scheme",
                "mutable": false,
                "complexType": {
                    "original": "ColorScheme",
                    "resolved": "\"dark\" | \"light\" | \"system\"",
                    "references": {
                        "ColorScheme": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ColorScheme"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Component color schema"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "'light'"
            },
            "index": {
                "type": "unknown",
                "attribute": "index",
                "mutable": false,
                "complexType": {
                    "original": "CloudIndexConfig | CloudIndexConfig[]",
                    "resolved": "CloudIndexConfig[] | { api_key: string; endpoint: string; }",
                    "references": {
                        "CloudIndexConfig": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::CloudIndexConfig"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Orama Index configuration\n\nnote: It will be overrided by clientInstance property"
                },
                "getter": false,
                "setter": false
            },
            "clientInstance": {
                "type": "unknown",
                "attribute": "client-instance",
                "mutable": false,
                "complexType": {
                    "original": "OramaClient | AnyOrama",
                    "resolved": "FunctionComponents<any> & Internals<any, AnyIndexStore, AnyDocumentStore, AnySorterStore> & ArrayCallbackComponents<any> & OramaID & { plugins: OramaPlugin<unknown>[]; } | OramaClient<true>",
                    "references": {
                        "OramaClient": {
                            "location": "import",
                            "path": "@oramacloud/client",
                            "id": "../../node_modules/.pnpm/@oramacloud+client@2.1.4/node_modules/@oramacloud/client/dist/index.d.ts::OramaClient"
                        },
                        "AnyOrama": {
                            "location": "import",
                            "path": "@orama/orama",
                            "id": "../../node_modules/.pnpm/@orama+orama@3.1.6/node_modules/@orama/orama/dist/commonjs/index.d.ts::AnyOrama"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Orama Instance or CollectionManager"
                },
                "getter": false,
                "setter": false
            },
            "oramaCoreClientInstance": {
                "type": "unknown",
                "attribute": "orama-core-client-instance",
                "mutable": false,
                "complexType": {
                    "original": "CollectionManager",
                    "resolved": "CollectionManager",
                    "references": {
                        "CollectionManager": {
                            "location": "import",
                            "path": "@orama/core",
                            "id": "../../node_modules/.pnpm/@orama+core@0.1.8/node_modules/@orama/core/script/index.d.ts::CollectionManager"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "open": {
                "type": "boolean",
                "attribute": "open",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": true,
                "defaultValue": "false"
            },
            "facetProperty": {
                "type": "string",
                "attribute": "facet-property",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Index result property to"
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "resultMap": {
                "type": "unknown",
                "attribute": "result-map",
                "mutable": false,
                "complexType": {
                    "original": "Partial<ResultMap>",
                    "resolved": "ResultMapItem[] | { path?: string | ResultMapRenderFunction; description?: string | ResultMapRenderFunction; title?: string | ResultMapRenderFunction; section?: string | ResultMapRenderFunction; datasourceId?: string | ResultMapRenderFunction; icon?: string | ResultItemRenderFunction; }",
                    "references": {
                        "Partial": {
                            "location": "global",
                            "id": "global::Partial"
                        },
                        "ResultMap": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ResultMap"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Used to map dataset result properties to the expected SearchBox properties"
                },
                "getter": false,
                "setter": false,
                "defaultValue": "{}"
            },
            "resultItemRender": {
                "type": "unknown",
                "attribute": "result-item-render",
                "mutable": false,
                "complexType": {
                    "original": "ResultItemRenderFunction",
                    "resolved": "(item: any, datasourceId: string) => string",
                    "references": {
                        "ResultItemRenderFunction": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ResultItemRenderFunction"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Used to render a custom icom per result. It should return a local asset path."
                },
                "getter": false,
                "setter": false
            },
            "sourceBaseUrl": {
                "type": "string",
                "attribute": "source-base-url",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Used to provide source base URL for the Search Results"
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "linksTarget": {
                "type": "string",
                "attribute": "links-target",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Used to provide linkRel to search result links"
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "linksRel": {
                "type": "string",
                "attribute": "links-rel",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Used to provide linkRel to search result links"
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "sourcesMap": {
                "type": "unknown",
                "attribute": "sources-map",
                "mutable": false,
                "complexType": {
                    "original": "SourcesMap",
                    "resolved": "SourcesMapItem[] | { path?: string | NewType; description?: string | NewType; title?: string | NewType; } & { datasourceId?: string; }",
                    "references": {
                        "SourcesMap": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::SourcesMap"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Used to map Chat result sources to expected Orama Chat properties"
                },
                "getter": false,
                "setter": false
            },
            "disableChat": {
                "type": "boolean",
                "attribute": "disable-chat",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Disables chat capabilities"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "false"
            },
            "layout": {
                "type": "string",
                "attribute": "layout",
                "mutable": false,
                "complexType": {
                    "original": "'modal' | 'embed'",
                    "resolved": "\"embed\" | \"modal\"",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "This component can behave either as Modal or a Embed component.\nFor Modal, a new absolute panel will be displayed on top.\nFor Embed, Orama Search Box will be displayed as a inline component."
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "'modal'"
            },
            "highlightTitle": {
                "type": "boolean",
                "attribute": "highlight-title",
                "mutable": false,
                "complexType": {
                    "original": "HighlightOptions | false",
                    "resolved": "HighlightOptions | boolean",
                    "references": {
                        "HighlightOptions": {
                            "location": "import",
                            "path": "@orama/highlight",
                            "id": "../../node_modules/.pnpm/@orama+highlight@0.1.9/node_modules/@orama/highlight/dist/index.d.ts::HighlightOptions"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Options for highlights of Search Result titles"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "false"
            },
            "highlightDescription": {
                "type": "boolean",
                "attribute": "highlight-description",
                "mutable": false,
                "complexType": {
                    "original": "HighlightOptions | false",
                    "resolved": "HighlightOptions | boolean",
                    "references": {
                        "HighlightOptions": {
                            "location": "import",
                            "path": "@orama/highlight",
                            "id": "../../node_modules/.pnpm/@orama+highlight@0.1.9/node_modules/@orama/highlight/dist/index.d.ts::HighlightOptions"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Options for highlights of Search Result descriptions"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "false"
            },
            "prompt": {
                "type": "string",
                "attribute": "prompt",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Prompt for the search box"
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "chatPlaceholder": {
                "type": "string",
                "attribute": "chat-placeholder",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Placeholder for chat input"
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "searchPlaceholder": {
                "type": "string",
                "attribute": "search-placeholder",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Placeholder for search input"
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "suggestions": {
                "type": "unknown",
                "attribute": "suggestions",
                "mutable": false,
                "complexType": {
                    "original": "string[]",
                    "resolved": "string[]",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "List of suggestions to show when the input is empty"
                },
                "getter": false,
                "setter": false
            },
            "searchParams": {
                "type": "unknown",
                "attribute": "search-params",
                "mutable": false,
                "complexType": {
                    "original": "SearchParams<Orama<AnyOrama | OramaClient>>",
                    "resolved": "SearchParamsFullText<Orama<AnyOrama | OramaClient<true>>, never> | SearchParamsHybrid<Orama<AnyOrama | OramaClient<true>>, never> | SearchParamsVector<Orama<AnyOrama | OramaClient<true>>, never>",
                    "references": {
                        "SearchParams": {
                            "location": "import",
                            "path": "@orama/orama",
                            "id": "../../node_modules/.pnpm/@orama+orama@3.1.6/node_modules/@orama/orama/dist/commonjs/index.d.ts::SearchParams"
                        },
                        "Orama": {
                            "location": "import",
                            "path": "@orama/orama",
                            "id": "../../node_modules/.pnpm/@orama+orama@3.1.6/node_modules/@orama/orama/dist/commonjs/index.d.ts::Orama"
                        },
                        "AnyOrama": {
                            "location": "import",
                            "path": "@orama/orama",
                            "id": "../../node_modules/.pnpm/@orama+orama@3.1.6/node_modules/@orama/orama/dist/commonjs/index.d.ts::AnyOrama"
                        },
                        "OramaClient": {
                            "location": "import",
                            "path": "@oramacloud/client",
                            "id": "../../node_modules/.pnpm/@oramacloud+client@2.1.4/node_modules/@oramacloud/client/dist/index.d.ts::OramaClient"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Parameters forwarded to Orama Client."
                },
                "getter": false,
                "setter": false
            },
            "chatMarkdownLinkTitle": {
                "type": "unknown",
                "attribute": "chat-markdown-link-title",
                "mutable": false,
                "complexType": {
                    "original": "ChatMarkdownLinkTitle",
                    "resolved": "({ text, href }: { text: string; href: string; }) => string",
                    "references": {
                        "ChatMarkdownLinkTitle": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ChatMarkdownLinkTitle"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Callback function used on every AI Chat link title"
                },
                "getter": false,
                "setter": false
            },
            "chatMarkdownLinkHref": {
                "type": "unknown",
                "attribute": "chat-markdown-link-href",
                "mutable": false,
                "complexType": {
                    "original": "ChatMarkdownLinkHref",
                    "resolved": "({ text, href }: { text: string; href: string; }) => string",
                    "references": {
                        "ChatMarkdownLinkHref": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ChatMarkdownLinkHref"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Callback function used on every AI Chat link"
                },
                "getter": false,
                "setter": false
            },
            "clearChatOnDisconnect": {
                "type": "boolean",
                "attribute": "clear-chat-on-disconnect",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Clear chat on disconnect"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "true"
            },
            "systemPrompts": {
                "type": "unknown",
                "attribute": "system-prompts",
                "mutable": false,
                "complexType": {
                    "original": "string[]",
                    "resolved": "string[]",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "System prompts to be used for the chat"
                },
                "getter": false,
                "setter": false
            },
            "chatMarkdownLinkTarget": {
                "type": "unknown",
                "attribute": "chat-markdown-link-target",
                "mutable": false,
                "complexType": {
                    "original": "ChatMarkdownLinkTarget",
                    "resolved": "({ text, href }: { text: string; href: string; }) => string",
                    "references": {
                        "ChatMarkdownLinkTarget": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ChatMarkdownLinkTarget"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Callback function used on every AI Chat link target"
                },
                "getter": false,
                "setter": false
            },
            "showKeyboardShortcuts": {
                "type": "boolean",
                "attribute": "show-keyboard-shortcuts",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Show keyboard shortcuts in the footer"
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "true"
            },
            "disclaimer": {
                "type": "string",
                "attribute": "disclaimer",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Disclaimer text to show below the chat input"
                },
                "getter": false,
                "setter": false,
                "reflect": false
            },
            "dictionary": {
                "type": "unknown",
                "attribute": "dictionary",
                "mutable": false,
                "complexType": {
                    "original": "Partial<Dictionary>",
                    "resolved": "{ searchPlaceholder?: string; chatPlaceholder?: string; noResultsFound?: string; noResultsFoundFor?: string; suggestions?: string; seeAll?: string; addMore?: string; clearChat?: string; errorMessage?: string; disclaimer?: string; startYourSearch?: string; initErrorSearch?: string; initErrorChat?: string; chatButtonLabel?: string; searchButtonLabel?: string; }",
                    "references": {
                        "Partial": {
                            "location": "global",
                            "id": "global::Partial"
                        },
                        "Dictionary": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::Dictionary"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "example",
                            "text": "const searchBox = document.querySelector('orama-search-box');\nsearchBox.dictionary = { searchPlaceholder: \"Search our products...\" };"
                        }],
                    "text": "Custom text dictionary for localization"
                },
                "getter": false,
                "setter": false,
                "defaultValue": "{}"
            },
            "relatedQueries": {
                "type": "number",
                "attribute": "related-queries",
                "mutable": false,
                "complexType": {
                    "original": "number",
                    "resolved": "number",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Display automatic chat suggestions"
                },
                "getter": false,
                "setter": false,
                "reflect": false
            }
        };
    }
    static get states() {
        return {
            "internalTextDictionary": {},
            "componentID": {},
            "systemScheme": {},
            "windowWidth": {}
        };
    }
    static get events() {
        return [{
                "method": "searchCompleted",
                "name": "searchCompleted",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when search successfully resolves"
                },
                "complexType": {
                    "original": "OnSearchCompletedCallbackProps",
                    "resolved": "{ clientSearchParams: ClientSearchParams; result: { results: SearchResultBySection[]; resultsCount: number; facets: Facet[]; }; }",
                    "references": {
                        "OnSearchCompletedCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::OnSearchCompletedCallbackProps"
                        }
                    }
                }
            }, {
                "method": "searchResultClick",
                "name": "searchResultClick",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when user clicks on search result"
                },
                "complexType": {
                    "original": "OnSearchResultClickCallbackProps",
                    "resolved": "{ result: SearchResult; }",
                    "references": {
                        "OnSearchResultClickCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::OnSearchResultClickCallbackProps"
                        }
                    }
                }
            }, {
                "method": "clearChat",
                "name": "clearChat",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when the chat is cleared"
                },
                "complexType": {
                    "original": "void",
                    "resolved": "void",
                    "references": {}
                }
            }, {
                "method": "startConversation",
                "name": "startConversation",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired as soon as the conversation is started"
                },
                "complexType": {
                    "original": "onStartConversationCallbackProps",
                    "resolved": "{ userPrompt: string; systemPrompts: string[]; }",
                    "references": {
                        "onStartConversationCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::onStartConversationCallbackProps"
                        }
                    }
                }
            }, {
                "method": "answerGenerated",
                "name": "answerGenerated",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when answer generation is successfully completed"
                },
                "complexType": {
                    "original": "OnAnswerGeneratedCallbackProps",
                    "resolved": "{ askParams: AskParams; query: string; sources: Results<unknown>; answer: string; segment: string; trigger: string; }",
                    "references": {
                        "OnAnswerGeneratedCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::OnAnswerGeneratedCallbackProps"
                        }
                    }
                }
            }, {
                "method": "answerSourceClick",
                "name": "answerSourceClick",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when user clicks on answer source"
                },
                "complexType": {
                    "original": "OnAnswerSourceClickCallbackProps",
                    "resolved": "{ source: SearchResult; }",
                    "references": {
                        "OnAnswerSourceClickCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::OnAnswerSourceClickCallbackProps"
                        }
                    }
                }
            }, {
                "method": "chatMarkdownLinkClicked",
                "name": "chatMarkdownLinkClicked",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when user clicks on chat markdown link"
                },
                "complexType": {
                    "original": "OnChatMarkdownLinkClickedCallbackProps",
                    "resolved": "{ text: string; href: string; }",
                    "references": {
                        "OnChatMarkdownLinkClickedCallbackProps": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::OnChatMarkdownLinkClickedCallbackProps"
                        }
                    }
                }
            }, {
                "method": "modalClosed",
                "name": "modalClosed",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Fired when modal is closed"
                },
                "complexType": {
                    "original": "any",
                    "resolved": "any",
                    "references": {}
                }
            }, {
                "method": "modalStatusChanged",
                "name": "modalStatusChanged",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [{
                            "name": "deprecated",
                            "text": "use modalClosed instead"
                        }],
                    "text": "Fire when modal status changes"
                },
                "complexType": {
                    "original": "{ open: boolean; id: HTMLElement }",
                    "resolved": "{ open: boolean; id: HTMLElement; }",
                    "references": {
                        "HTMLElement": {
                            "location": "global",
                            "id": "global::HTMLElement"
                        }
                    }
                }
            }];
    }
    static get elementRef() { return "htmlElement"; }
    static get watchers() {
        return [{
                "propName": "dictionary",
                "methodName": "handleDictionaryChange"
            }, {
                "propName": "index",
                "methodName": "indexChanged"
            }, {
                "propName": "clientInstance",
                "methodName": "indexChanged"
            }, {
                "propName": "oramaCoreClientInstance",
                "methodName": "indexChanged"
            }, {
                "propName": "themeConfig",
                "methodName": "watchHandler"
            }, {
                "propName": "colorScheme",
                "methodName": "watchHandler"
            }, {
                "propName": "open",
                "methodName": "handleOpenPropChange"
            }, {
                "propName": "facetProperty",
                "methodName": "handleFacetPropertyChange"
            }, {
                "propName": "searchParams",
                "methodName": "handleSearchParamsChange"
            }];
    }
    static get listeners() {
        return [{
                "name": "keydown",
                "method": "handleKeyDown",
                "target": "document",
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=orama-search-box.js.map
