import { h, Host } from "@stencil/core";
import "@phosphor-icons/webcomponents/dist/icons/PhMagnifyingGlass.mjs";
import { generateRandomID, updateCssVariables, updateThemeClasses } from "../../utils/utils";
import { getText as getTextUtil } from "../../utils/textDictionary";
export class OramaSearchButton {
    constructor() {
        this.size = 'medium';
        this.colorScheme = 'light';
        /**
         * Text dictionary for customizing all text content in the component.
         * This can be set either via HTML attribute as a JSON string or via JavaScript as an object.
         * @example
         * // Via HTML attribute
         * <orama-search-button dictionary='{"searchButtonLabel": "Search docs"}' />
         *
         * // Via JavaScript
         * const searchButton = document.querySelector('orama-search-button');
         * searchButton.dictionary = { searchButtonLabel: "Search docs" };
         */
        this.dictionary = {};
        this.systemScheme = 'light';
        this.shortcutLabel = '';
        this.componentID = generateRandomID('search-button');
        this.onPrefersColorSchemeChange = (event) => {
            this.systemScheme = event.matches ? 'dark' : 'light';
            this.updateTheme();
        };
    }
    /**
     * Watch for changes to the dictionary prop
     */
    handleTextDictionaryChange(newValue) {
        // Handle case where dictionary is passed as a string (via HTML attribute)
        if (typeof newValue === 'string') {
            try {
                this.dictionary = JSON.parse(newValue);
            }
            catch (e) {
                console.error('Error parsing dictionary:', e);
            }
        }
    }
    /**
     * Gets the text for a specific key from the dictionary prop.
     *
     * @param key - The key to get the text for
     * @returns The text for the specified key
     */
    getText(key) {
        return getTextUtil(key, this.dictionary);
    }
    watchHandler() {
        this.updateTheme();
    }
    handleSearchboxClosed(event) {
        // TODO: should be based on the id of current searchbox
        this.buttonRef.querySelector('button').focus();
    }
    // trigger click when entering  '⌘ K' o Mac or 'Ctrl + K'
    handleKeyDown(event) {
        if (event.key === 'k' && (event.metaKey || event.ctrlKey)) {
            event.preventDefault();
            this.buttonRef.click();
        }
    }
    updateTheme() {
        const scheme = updateThemeClasses(this.htmlElement, this.colorScheme, this.systemScheme);
        updateCssVariables(this.htmlElement, scheme, this.themeConfig);
    }
    handleShortcutLabel() {
        const userAgent = navigator.userAgent;
        const isMac = userAgent.includes('Mac');
        return isMac ? '⌘ K' : 'Ctrl + K';
    }
    connectedCallback() {
        this.htmlElement.id = this.componentID;
        this.shortcutLabel = this.handleShortcutLabel();
        this.schemaQuery = window.matchMedia('(prefers-color-scheme: dark)');
        this.systemScheme = this.schemaQuery.matches ? 'dark' : 'light';
        this.updateTheme();
        this.schemaQuery.addEventListener('change', this.onPrefersColorSchemeChange);
    }
    disconnectedCallback() {
        this.schemaQuery.removeEventListener('change', this.onPrefersColorSchemeChange);
    }
    componentWillLoad() {
        this.updateTheme();
    }
    render() {
        return (h(Host, { key: 'd784308be3ef8bb2abeaa25bed43f51962ec7586' }, h("orama-button", { key: '2dafcd15bb0b5f468e6a91d9651cec22f2670a61', type: "button", variant: "secondary", ref: (el) => (this.buttonRef = el), size: this.size, onClick: () => {
                const searchBoxList = document.getElementsByTagName('orama-search-box');
                if (!searchBoxList.length) {
                    console.error('No OramaSearchBox found');
                    return;
                }
                if (searchBoxList.length > 1) {
                    console.error('Multiple instances of OramaSearchBox (orama-search-box) found. Considere to assign an ID to OramaSearchBox, implement onClick on OramaSearchButton and select desired OramaSearchBox on onClick method');
                    return;
                }
                searchBoxList[0].open = true;
            } }, h("span", { key: 'c96898d75e57d546930794c25d04a6cd3cd3f960', slot: "adorment-start" }, h("ph-magnifying-glass", { key: 'a42c0d765c8bdf1647ebc7d9e85c8ce224397ddf' })), h("slot", { key: '979f16ad8739765dc42ed9cbb46d6b2eb789e000' }, this.getText('searchButtonLabel')), h("span", { key: 'e211d3ccd39bbe8b628569a4d69ff23c7a0a9935', slot: "adorment-end", class: "kyb-shortcut" }, this.shortcutLabel))));
    }
    static get is() { return "orama-search-button"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() {
        return {
            "$": ["orama-search-button.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["orama-search-button.css"]
        };
    }
    static get properties() {
        return {
            "size": {
                "type": "string",
                "attribute": "size",
                "mutable": false,
                "complexType": {
                    "original": "'small' | 'medium' | 'large'",
                    "resolved": "\"large\" | \"medium\" | \"small\"",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "'medium'"
            },
            "themeConfig": {
                "type": "unknown",
                "attribute": "theme-config",
                "mutable": false,
                "complexType": {
                    "original": "Partial<TThemeOverrides>",
                    "resolved": "{ typography?: DeepPartial<{ '--font-primary': string; }>; colors?: DeepPartial<{ gray50: string; gray100: string; gray200: string; gray300: string; gray400: string; gray500: string; gray600: string; gray700: string; gray800: string; gray900: string; gray950: string; purple100: string; purple200: string; purple300: string; purple500: string; purple600: string; purple700: string; light: { \"--text-color-primary\": string; \"--text-color-secondary\": string; \"--text-color-tertiary\": string; \"--text-color-accent\": string; \"--text-color-inactive\": string; \"--text-color-reverse\": string; \"--background-color-primary\": string; \"--background-color-secondary\": string; \"--background-color-tertiary\": string; \"--background-color-fourth\": string; \"--background-color-reverse\": string; \"--background-color-accent\": string; \"--border-color-primary\": string; \"--border-color-secondary\": string; \"--border-color-tertiary\": string; \"--border-color-accent\": string; \"--icon-color-primary\": string; \"--icon-color-secondary\": string; \"--icon-color-tertiary\": string; \"--icon-color-inactive\": string; \"--icon-color-reverse\": string; \"--shadow-color-primary\": string; \"--button-text-color-primary\": string; \"--button-text-color-secondary\": string; \"--button-background-color-primary\": string; \"--button-background-color-secondary\": string; \"--button-background-color-secondary-hover\": string; \"--button-border-color-secondary\": string; \"--backdrop-background-color-primary\": string; \"--chat-button-border-color-gradientOne\": string; \"--chat-button-border-color-gradientTwo\": string; \"--chat-button-border-color-gradientThree\": string; \"--chat-button-border-color-gradientFour\": string; \"--chat-button-border-color-gradientFive\": string; \"--chat-button-border-color-gradientSix\": string; \"--chat-button-background-color-gradientOne\": string; \"--chat-button-background-color-gradientTwo\": string; }; dark: { \"--text-color-primary\": string; \"--text-color-secondary\": string; \"--text-color-tertiary\": string; \"--text-color-accent\": string; \"--text-color-inactive\": string; \"--text-color-reverse\": string; \"--background-color-primary\": string; \"--background-color-secondary\": string; \"--background-color-tertiary\": string; \"--background-color-fourth\": string; \"--background-color-reverse\": string; \"--background-color-accent\": string; \"--border-color-primary\": string; \"--border-color-secondary\": string; \"--border-color-tertiary\": string; \"--border-color-accent\": string; \"--icon-color-primary\": string; \"--icon-color-secondary\": string; \"--icon-color-tertiary\": string; \"--icon-color-inactive\": string; \"--icon-color-reverse\": string; \"--shadow-color-primary\": string; \"--button-text-color-primary\": string; \"--button-text-color-secondary\": string; \"--button-background-color-primary\": string; \"--button-background-color-secondary\": string; \"--button-background-color-secondary-hover\": string; \"--button-border-color-secondary\": string; \"--backdrop-background-color-primary\": string; \"--chat-button-border-color-gradientOne\": string; \"--chat-button-border-color-gradientTwo\": string; \"--chat-button-border-color-gradientThree\": string; \"--chat-button-border-color-gradientFour\": string; \"--chat-button-border-color-gradientFive\": string; \"--chat-button-border-color-gradientSix\": string; \"--chat-button-background-color-gradientOne\": string; \"--chat-button-background-color-gradientTwo\": string; }; }>; radius?: DeepPartial<{ '--radius-xs': string; '--radius-s': string; '--radius-m': string; '--radius-l': string; '--radius-3xl': string; '--textarea-radius': string; }>; shadow?: DeepPartial<{ '--textarea-shadow': any; }>; }",
                    "references": {
                        "Partial": {
                            "location": "global",
                            "id": "global::Partial"
                        },
                        "TThemeOverrides": {
                            "location": "import",
                            "path": "@/components",
                            "id": "src/components.d.ts::TThemeOverrides"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false
            },
            "colorScheme": {
                "type": "string",
                "attribute": "color-scheme",
                "mutable": false,
                "complexType": {
                    "original": "ColorScheme",
                    "resolved": "\"dark\" | \"light\" | \"system\"",
                    "references": {
                        "ColorScheme": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::ColorScheme"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": ""
                },
                "getter": false,
                "setter": false,
                "reflect": false,
                "defaultValue": "'light'"
            },
            "dictionary": {
                "type": "unknown",
                "attribute": "dictionary",
                "mutable": false,
                "complexType": {
                    "original": "Partial<Dictionary>",
                    "resolved": "{ searchPlaceholder?: string; chatPlaceholder?: string; noResultsFound?: string; noResultsFoundFor?: string; suggestions?: string; seeAll?: string; addMore?: string; clearChat?: string; errorMessage?: string; disclaimer?: string; startYourSearch?: string; initErrorSearch?: string; initErrorChat?: string; chatButtonLabel?: string; searchButtonLabel?: string; }",
                    "references": {
                        "Partial": {
                            "location": "global",
                            "id": "global::Partial"
                        },
                        "Dictionary": {
                            "location": "import",
                            "path": "@/types",
                            "id": "src/types/index.ts::Dictionary"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "example",
                            "text": "// Via HTML attribute\n<orama-search-button dictionary='{\"searchButtonLabel\": \"Search docs\"}' />\n\n// Via JavaScript\nconst searchButton = document.querySelector('orama-search-button');\nsearchButton.dictionary = { searchButtonLabel: \"Search docs\" };"
                        }],
                    "text": "Text dictionary for customizing all text content in the component.\nThis can be set either via HTML attribute as a JSON string or via JavaScript as an object."
                },
                "getter": false,
                "setter": false,
                "defaultValue": "{}"
            }
        };
    }
    static get states() {
        return {
            "systemScheme": {},
            "shortcutLabel": {},
            "componentID": {}
        };
    }
    static get elementRef() { return "htmlElement"; }
    static get watchers() {
        return [{
                "propName": "dictionary",
                "methodName": "handleTextDictionaryChange"
            }, {
                "propName": "themeConfig",
                "methodName": "watchHandler"
            }, {
                "propName": "colorScheme",
                "methodName": "watchHandler"
            }];
    }
    static get listeners() {
        return [{
                "name": "searchboxClosed",
                "method": "handleSearchboxClosed",
                "target": "body",
                "capture": false,
                "passive": false
            }, {
                "name": "keydown",
                "method": "handleKeyDown",
                "target": "document",
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=orama-search-button.js.map
