import { e as getRenderingRef, f as forceUpdate, i as getElement } from './p-BGpMMtbT.js';

const ChatStoreInitialProps = {
    chatService: null,
    interactions: [],
    sourceBaseURL: '',
    linksTarget: '_blank',
    linksRel: 'noopener noreferrer',
    prompt: '',
    sourcesMap: {
        title: 'title',
        description: 'description',
        path: 'path',
    },
};

const appendToMap = (map, propName, value) => {
    const items = map.get(propName);
    if (!items) {
        map.set(propName, [value]);
    }
    else if (!items.includes(value)) {
        items.push(value);
    }
};
const debounce = (fn, ms) => {
    let timeoutId;
    return (...args) => {
        if (timeoutId) {
            clearTimeout(timeoutId);
        }
        timeoutId = setTimeout(() => {
            timeoutId = 0;
            fn(...args);
        }, ms);
    };
};

/**
 * Check if a possible element isConnected.
 * The property might not be there, so we check for it.
 *
 * We want it to return true if isConnected is not a property,
 * otherwise we would remove these elements and would not update.
 *
 * Better leak in Edge than to be useless.
 */
const isConnected = (maybeElement) => !('isConnected' in maybeElement) || maybeElement.isConnected;
const cleanupElements = debounce((map) => {
    for (let key of map.keys()) {
        map.set(key, map.get(key).filter(isConnected));
    }
}, 2_000);
const stencilSubscription = () => {
    if (typeof getRenderingRef !== 'function') {
        // If we are not in a stencil project, we do nothing.
        // This function is not really exported by @stencil/core.
        return {};
    }
    const elmsToUpdate = new Map();
    return {
        dispose: () => elmsToUpdate.clear(),
        get: (propName) => {
            const elm = getRenderingRef();
            if (elm) {
                appendToMap(elmsToUpdate, propName, elm);
            }
        },
        set: (propName) => {
            const elements = elmsToUpdate.get(propName);
            if (elements) {
                elmsToUpdate.set(propName, elements.filter(forceUpdate));
            }
            cleanupElements(elmsToUpdate);
        },
        reset: () => {
            elmsToUpdate.forEach((elms) => elms.forEach(forceUpdate));
            cleanupElements(elmsToUpdate);
        },
    };
};

const unwrap = (val) => (typeof val === 'function' ? val() : val);
const createObservableMap = (defaultState, shouldUpdate = (a, b) => a !== b) => {
    const unwrappedState = unwrap(defaultState);
    let states = new Map(Object.entries(unwrappedState ?? {}));
    const handlers = {
        dispose: [],
        get: [],
        set: [],
        reset: [],
    };
    const reset = () => {
        // When resetting the state, the default state may be a function - unwrap it to invoke it.
        // otherwise, the state won't be properly reset
        states = new Map(Object.entries(unwrap(defaultState) ?? {}));
        handlers.reset.forEach((cb) => cb());
    };
    const dispose = () => {
        // Call first dispose as resetting the state would
        // cause less updates ;)
        handlers.dispose.forEach((cb) => cb());
        reset();
    };
    const get = (propName) => {
        handlers.get.forEach((cb) => cb(propName));
        return states.get(propName);
    };
    const set = (propName, value) => {
        const oldValue = states.get(propName);
        if (shouldUpdate(value, oldValue, propName)) {
            states.set(propName, value);
            handlers.set.forEach((cb) => cb(propName, value, oldValue));
        }
    };
    const state = (typeof Proxy === 'undefined'
        ? {}
        : new Proxy(unwrappedState, {
            get(_, propName) {
                return get(propName);
            },
            ownKeys(_) {
                return Array.from(states.keys());
            },
            getOwnPropertyDescriptor() {
                return {
                    enumerable: true,
                    configurable: true,
                };
            },
            has(_, propName) {
                return states.has(propName);
            },
            set(_, propName, value) {
                set(propName, value);
                return true;
            },
        }));
    const on = (eventName, callback) => {
        handlers[eventName].push(callback);
        return () => {
            removeFromArray(handlers[eventName], callback);
        };
    };
    const onChange = (propName, cb) => {
        const unSet = on('set', (key, newValue) => {
            if (key === propName) {
                cb(newValue);
            }
        });
        // We need to unwrap the defaultState because it might be a function.
        // Otherwise we might not be sending the right reset value.
        const unReset = on('reset', () => cb(unwrap(defaultState)[propName]));
        return () => {
            unSet();
            unReset();
        };
    };
    const use = (...subscriptions) => {
        const unsubs = subscriptions.reduce((unsubs, subscription) => {
            if (subscription.set) {
                unsubs.push(on('set', subscription.set));
            }
            if (subscription.get) {
                unsubs.push(on('get', subscription.get));
            }
            if (subscription.reset) {
                unsubs.push(on('reset', subscription.reset));
            }
            if (subscription.dispose) {
                unsubs.push(on('dispose', subscription.dispose));
            }
            return unsubs;
        }, []);
        return () => unsubs.forEach((unsub) => unsub());
    };
    const forceUpdate = (key) => {
        const oldValue = states.get(key);
        handlers.set.forEach((cb) => cb(key, oldValue, oldValue));
    };
    return {
        state,
        get,
        set,
        on,
        onChange,
        use,
        dispose,
        reset,
        forceUpdate,
    };
};
const removeFromArray = (array, item) => {
    const index = array.indexOf(item);
    if (index >= 0) {
        array[index] = array[array.length - 1];
        array.length--;
    }
};

const createStore = (defaultState, shouldUpdate) => {
    const map = createObservableMap(defaultState, shouldUpdate);
    map.use(stencilSubscription());
    return map;
};

const GlobalStoreInitialProps = {
    open: false,
    currentTask: 'search',
    currentTerm: '',
};
createStore(GlobalStoreInitialProps);

const SearchStoreInitialProps = {
    count: 0,
    facets: [],
    facetProperty: '', // TODO: consider to move to resultsMap
    results: [],
    resultMap: {},
    highlightedIndex: -1,
    loading: false,
    error: false,
    // TODO: Probable needs to be held in component property.
    // Lets queckly dicudd about this again.
    searchService: null,
    searchParams: null,
};

// biome-ignore lint/suspicious/noExplicitAny: Any is acceptable here
const storesMapInitialProps = {
    global: GlobalStoreInitialProps,
    search: SearchStoreInitialProps,
    chat: ChatStoreInitialProps,
};
/**
 * Holds the global references for all stores in Orama Ui.
 *
 */
const storesReferencesMap = new Map();
/**
 * Access storeSReferenceMap and return the initialized store. Throw exception if either parent or store does not exist
 *
 * @param parentComponentId
 * @param storeName
 * @returns
 */
const getParentComponentStore = (parentComponentId, storeName) => {
    const parentStores = storesReferencesMap.get(parentComponentId);
    if (!parentStores) {
        throw new Error('Invalid parent component Id');
    }
    const store = parentStores[storeName];
    if (!store) {
        throw new Error('Store not initialized');
    }
    return store;
};
const initStore = (storeName, parentComponentId) => {
    const initialProps = storesMapInitialProps[storeName];
    if (!initialProps) {
        throw new Error('Invalid store name');
    }
    const store = createStore(initialProps);
    if (storesReferencesMap.has(parentComponentId)) {
        const currentStores = storesReferencesMap.get(parentComponentId);
        currentStores[storeName] = store;
    }
    else {
        storesReferencesMap.set(parentComponentId, { [storeName]: store });
    }
    return store;
};
const removeAllStores = (parentComponentId) => {
    storesReferencesMap.delete(parentComponentId);
};
function getStore(storeName, element) {
    const externalComponent = getExternalParentComponentHTMLElement(element);
    if (!externalComponent) {
        throw new Error('Failed to get store');
    }
    return getParentComponentStore(externalComponent.id, storeName);
}
const EXTERNAL_COMPONENT_TAG_LIST = ['orama-search-box', 'orama-chat-box'];
function getExternalParentComponentHTMLElement(element) {
    var _a;
    let currentNode = element;
    while (true) {
        if (!currentNode) {
            return null;
        }
        if (currentNode instanceof ShadowRoot) {
            const host = currentNode.host;
            if (EXTERNAL_COMPONENT_TAG_LIST.includes(host.tagName.toLowerCase())) {
                return host;
            }
            currentNode = host;
        }
        else {
            currentNode = (_a = currentNode.parentNode) !== null && _a !== void 0 ? _a : null;
        }
    }
}

// Define unique symbols for internal metadata
const STORE_PROPS = Symbol('storeProps');
const STORE_WILL_LOAD_PATCHED = Symbol('storeWillLoadPatched');
/**
 * A decorator that:
 * - Patches `componentWillLoad` to initialize a Component Store.
 */
function Store(storeName) {
    return (targetClass, propKey) => {
        const classConstructor = targetClass.constructor;
        // Initialize the store properties metadata array if needed.
        if (!classConstructor[STORE_PROPS]) {
            classConstructor[STORE_PROPS] = [];
        }
        // Save both the property key and storeName in the metadata.
        classConstructor[STORE_PROPS].push({ propKey, storeName });
        // Patch componentWillLoad only once for the class.
        if (!classConstructor[STORE_WILL_LOAD_PATCHED]) {
            classConstructor[STORE_WILL_LOAD_PATCHED] = true;
            const originalComponentWillLoad = targetClass.componentWillLoad;
            // Replace componentWillLoad with our own wrapped version.
            targetClass.componentWillLoad = function () {
                // Access the host element
                const hostEl = getElement(this);
                const storeProps = this.constructor[STORE_PROPS];
                if (storeProps) {
                    for (const { propKey, storeName } of storeProps) {
                        const store = getStore(storeName, hostEl);
                        this[propKey] = store;
                    }
                }
                // Call the original componentWillLoad (if it exists).
                if (typeof originalComponentWillLoad === 'function') {
                    originalComponentWillLoad.apply(this);
                }
            };
        }
    };
}

export { Store as S, initStore as i, removeAllStores as r };
//# sourceMappingURL=p-9VPaspy8.js.map

//# sourceMappingURL=p-9VPaspy8.js.map