import { type EventEmitter } from '../../stencil-public-runtime';
import type { AnyOrama, Orama, SearchParams } from '@orama/orama';
import type { HighlightOptions } from '@orama/highlight';
import type { OramaClient } from '@oramacloud/client';
import type { CollectionManager } from '@orama/core';
import type { ChatMarkdownLinkHref, ChatMarkdownLinkTarget, ChatMarkdownLinkTitle, CloudIndexConfig, ColorScheme, OnAnswerGeneratedCallbackProps, OnAnswerSourceClickCallbackProps, OnChatMarkdownLinkClickedCallbackProps, OnSearchCompletedCallbackProps, OnSearchResultClickCallbackProps, onStartConversationCallbackProps, ResultItemRenderFunction, ResultMap, SourcesMap, Dictionary } from "../../types/index";
import type { TThemeOverrides } from "../../config/theme";
export declare class SearchBox {
    htmlElement: HTMLElement;
    /**
     * Component theme customization
     */
    themeConfig?: Partial<TThemeOverrides>;
    /**
     * Component color schema
     */
    colorScheme?: ColorScheme;
    /**
     * Orama Index configuration
     *
     * note: It will be overrided by clientInstance property
     */
    index?: CloudIndexConfig | CloudIndexConfig[];
    /**
     * Orama Instance or CollectionManager
     */
    clientInstance?: OramaClient | AnyOrama;
    oramaCoreClientInstance?: CollectionManager;
    open: boolean;
    /**
     * Index result property to
     */
    facetProperty?: string;
    /**
     * Used to map dataset result properties to the expected SearchBox properties
     */
    resultMap?: Partial<ResultMap>;
    /**
     * Used to render a custom icom per result. It should return a local asset path.
     */
    resultItemRender?: ResultItemRenderFunction;
    /**
     * Used to provide source base URL for the Search Results
     */
    sourceBaseUrl?: string;
    /**
     * Used to provide linkRel to search result links
     */
    linksTarget?: string;
    /**
     * Used to provide linkRel to search result links
     */
    linksRel?: string;
    /**
     * Used to map Chat result sources to expected Orama Chat properties
     */
    sourcesMap?: SourcesMap;
    /**
     * Disables chat capabilities
     */
    disableChat?: boolean;
    /**
     * This component can behave either as Modal or a Embed component.
     * For Modal, a new absolute panel will be displayed on top.
     * For Embed, Orama Search Box will be displayed as a inline component.
     */
    layout?: 'modal' | 'embed';
    /**
     * Options for highlights of Search Result titles
     */
    highlightTitle?: HighlightOptions | false;
    /**
     * Options for highlights of Search Result descriptions
     */
    highlightDescription?: HighlightOptions | false;
    /**
     * Prompt for the search box
     */
    prompt?: string;
    /**
     * Placeholder for chat input
     */
    chatPlaceholder?: string;
    /**
     * Placeholder for search input
     */
    searchPlaceholder?: string;
    /**
     * List of suggestions to show when the input is empty
     */
    suggestions?: string[];
    /**
     * Parameters forwarded to Orama Client.
     */
    searchParams?: SearchParams<Orama<AnyOrama | OramaClient>>;
    /**
     * Callback function used on every AI Chat link title
     */
    chatMarkdownLinkTitle?: ChatMarkdownLinkTitle;
    /**
     * Callback function used on every AI Chat link
     */
    chatMarkdownLinkHref?: ChatMarkdownLinkHref;
    /**
     * Clear chat on disconnect
     */
    clearChatOnDisconnect?: boolean;
    /**
     * System prompts to be used for the chat
     */
    systemPrompts?: string[];
    /**
     * Callback function used on every AI Chat link target
     */
    chatMarkdownLinkTarget?: ChatMarkdownLinkTarget;
    /**
     * Show keyboard shortcuts in the footer
     */
    showKeyboardShortcuts?: boolean;
    /**
     * Disclaimer text to show below the chat input
     */
    disclaimer?: string;
    /**
     * Custom text dictionary for localization
     * @example
     * const searchBox = document.querySelector('orama-search-box');
     * searchBox.dictionary = { searchPlaceholder: "Search our products..." };
     */
    dictionary?: Partial<Dictionary>;
    /**
     * Display automatic chat suggestions
     */
    relatedQueries?: number;
    /**
     * Watch for changes to the dictionary prop
     */
    handleDictionaryChange(newValue: Partial<Dictionary> | string): void;
    /**
     * Internal state for the parsed text dictionary.
     */
    private internalTextDictionary;
    componentID: string;
    systemScheme: Omit<ColorScheme, 'system'>;
    windowWidth: number;
    private searchStore;
    private chatStore;
    private globalStore;
    /**
     * Fired when search successfully resolves
     */
    searchCompleted: EventEmitter<OnSearchCompletedCallbackProps>;
    /**
     * Fired when user clicks on search result
     */
    searchResultClick: EventEmitter<OnSearchResultClickCallbackProps>;
    /**
     * Fired when the chat is cleared
     */
    clearChat: EventEmitter<void>;
    /**
     * Fired as soon as the conversation is started
     */
    startConversation: EventEmitter<onStartConversationCallbackProps>;
    /**
     * Fired when answer generation is successfully completed
     */
    answerGenerated: EventEmitter<OnAnswerGeneratedCallbackProps>;
    /**
     * Fired when user clicks on answer source
     */
    answerSourceClick: EventEmitter<OnAnswerSourceClickCallbackProps>;
    /**
     * Fired when user clicks on chat markdown link
     */
    chatMarkdownLinkClicked: EventEmitter<OnChatMarkdownLinkClickedCallbackProps>;
    /**
     * Fired when modal is closed
     */
    modalClosed: EventEmitter;
    /**
     * Fire when modal status changes
     * @deprecated use modalClosed instead
     */
    modalStatusChanged: EventEmitter<{
        open: boolean;
        id: HTMLElement;
    }>;
    wrapperRef: HTMLElement;
    schemaQuery: MediaQueryList;
    indexChanged(): void;
    watchHandler(): void;
    handleOpenPropChange(newValue: boolean): void;
    handleFacetPropertyChange(newValue: string): void;
    handleSearchParamsChange(newValue: SearchParams<Orama<AnyOrama | OramaClient>>): void;
    handleKeyDown(ev: KeyboardEvent): void;
    updateTheme(): void;
    getOldOramaClient(): AnyOrama | OramaClient<true>;
    bootstrap(): void;
    startServices(): void;
    componentWillLoad(): void;
    connectedCallback(): void;
    disconnectedCallback(): void;
    /**
     * Gets the text for a specific key from the dictionary prop.
     * Prioritizes direct props (searchPlaceholder, chatPlaceholder) for backward compatibility,
     * then falls back to the dictionary prop, and finally to the defaultTextDictionary.
     *
     * @param key - The key to get the text for
     * @returns The text for the specified key
     */
    getText(key: keyof Dictionary): string;
    getSearchBox(): any;
    getChatBox(): any;
    getInnerContent(): any;
    getOuterContent(): any;
    getModalLayout(): any;
    getEmbedLayout(): any;
    private closeSearchbox;
    private onChatButtonClick;
    private onPrefersColorSchemeChange;
    private updateWindowWidth;
    render(): any;
}
